// This software including the source code and documentation (the "SOFTWARE")
// is copyrighted.  You are free to use the SOFTWARE for any non-commercial
// purpose whatsoever provided you do not remove this copyright notice.
// If you wish to use the SOFTWARE for commercial purposes, please contact us
// to obtain a license.  The SOFTWARE is provided "as is", with absolutely no
// warranty.  Use it at your own risk.
// Website: http://www.math.ie/checker.html

// for the run-time configuration
int const SUDOKU_X         = 0x01;
int const SUDOKU_DISJGPS   = 0x02;
int const FIND_ALL_UNAVS   = 0x10;
int const INCLUDE_PSEUDOS  = 0x20;
int const PRINT_STATS      = 0x40;
// static variable to store above flags
static int s_flags = 0;

#define SetFlag( flag, bSet )    \
   if ( (bSet) )                 \
      s_flags |= (flag);         \
   else                          \
      s_flags &= (~(flag));

#define QueryFlag( flag )  ((s_flags & flag) == flag)


void EnableSudokuX( bool bEnable /* = true */ )
   {
   SetFlag( SUDOKU_X, bEnable );
   }

void EnableSudokuDG( bool bEnable /* = true */ )
   {
   SetFlag( SUDOKU_DISJGPS, bEnable );
   }

void EnableFindAllUnavoidables( bool bEnable /* = true */ )
   {
   SetFlag( FIND_ALL_UNAVS, bEnable );
   }

void EnableIncludePseudos( bool bEnable /* = true */ )
   {
   SetFlag( INCLUDE_PSEUDOS, bEnable );
   }

void EnableStats( bool bEnable /* = true */ )
   {
   SetFlag( PRINT_STATS, bEnable );
   }

bool DoWeWantPseudos()
   {
   return QueryFlag( INCLUDE_PSEUDOS );
   }

bool ShouldWeFindAllUnavoidables()
   {
   return QueryFlag( FIND_ALL_UNAVS );
   }

bool IsSudokuDG()
   {
   return QueryFlag( SUDOKU_DISJGPS );
   }

bool IsSudokuX()
   {
   return QueryFlag( SUDOKU_X );
   }

bool DoWeWantStats()
   {
   return QueryFlag( PRINT_STATS );
   }
