// This software including the source code and documentation (the "SOFTWARE")
// is copyrighted.  You are free to use the SOFTWARE for any non-commercial
// purpose whatsoever provided you do not remove this copyright notice.
// If you wish to use the SOFTWARE for commercial purposes, please contact us
// to obtain a license.  The SOFTWARE is provided "as is", with absolutely no
// warranty.  Use it at your own risk.
// Website: http://www.math.ie/checker.html

#include "unique.h"

// written by Guenter Stertenbrink, sterten@aol.com
// Website: http://magictour.free.fr/sudoku.htm
// modified for Sudoku-X by Bo Haglund

#include "config.h"
#include <memory.h>
#include <stdio.h>

static int smax = 2; // default: only check if a puzzle has a unique completion or not

int const N  = 3;
int const N2 = N * N;
int const N4 = N2 * N2;

static int Rows[ 5 * N4 + 1 ];
static int Row[ 5 * N4 + 1 ][ N2 + 1 ];
static int Cols[ N2 * N4 + 1 ];
static int Col[ N2 * N4 + 1 ][ 6 + 1 ];
static int Ur[ N2 * N4 + 1];
static int Uc[ 5 * N4 + 1 ];
static int V[ 5 * N4 + 1 ];
static int C[ N4 + 1 ];
static int I[ N4 + 1 ];
static int m;
static int const  n = N2 * N4;
static int A[ N4 + 1 ][ N4 + 1 ];

#define ZeroMemory( array ) memset( (array), 0, sizeof( array ) );

// distinguish between puzzles with 1, 2, or more than 2 completions
void EnablePseudos()
   {
   smax = 3; 
   }

// distinguish only between those puzzles completing uniquely
// and those not completing uniquely
void DisablePseudos()
   {
   smax = 2;
   }

void InitSolver()
   {
   /* Here starts initialization code. This code only needs to be executed the first time
   that the solver is called. If the solver then is repeatedly called, the initialization
   code can be lifted out and placed in a separate routine.*/
   m = 4*N4;
   if ( IsSudokuX() )
      m += 2*N2;
   else if ( IsSudokuDG() )
      m += N4;

   int r = 0;
   for ( int x = 1;  x <= N2;  x++ )
      for ( int y = 1; y <= N2;  y++ )
         for ( int s = 1;  s <= N2;  s++ )
            { 
            r++;
            Cols[r]=4;
            Col[r][1] = x*N2-N2+y;
            Col[r][2] = (N*((x-1)/N)+(y-1)/N)*N2+s+N4; 
            Col[r][3] = x*N2-N2+s+N4*2;
            Col[r][4] = y*N2-N2+s+N4*3;

            /* Following is additional code for adding columns for "Sudoku X" */
            if ( IsSudokuX() )
               {
               if ( x == y )
                  {
                  Col[r][5] = s + N4*4;
                  if ( x == (1 + N2-y) )
                     {
                     Cols[r]=6;
                     Col[r][6]=s+N2+N4*4;
                     }
                  else
                     Cols[r]=5;
                  } 
               else if ( x == (1 + N2 - y) )
                  {
                  Cols[r]=5;
                  Col[r][5]=s+N2+N4*4;
                  }
               } 
            else if ( IsSudokuDG() )
               {
               Cols[r] = 5;
               Col[r][5] = ((x-1)%3 + 3*((y-1)%3))*N2 + s + N4*4;
               }
            }

   ZeroMemory( Rows );
   for ( r = 1;  r <= n;  r++ )
      for ( int c = 1;  c <= Cols[ r ];  c++ )
         { 
         int x = Col[ r ][ c ];
         Rows[ x ]++;
         Row[ x ][ Rows[ x ] ] = r;
         }
   /* Here ends the initialization code. */ 
   } 

int GetNumberOfCompletionsOfPuzzle( char const* pszPuzzle )
   {
   int m0, m1, i, j, k, l, r, r1, c, c1, c2, x, y, min;
   int nodes = 0, clues = 0, solutions = -1;

#ifndef _WIN32
   // on Unix (for some reason) the solver needs to be initialised
   // each time we run a puzzle through it ...
   InitSolver();
#endif
   int ichPuzzle = 0;
   for ( i = 1;  i <= 9;  i++ )
      for ( j = 1;  j <= 9;  j++ )
         A[ i ][ j ] = pszPuzzle[ ichPuzzle++ ] - '0';

   ZeroMemory( Uc );
   ZeroMemory( Ur );
  for(x=1;x<=9;x++)for(y=1;y<=9;y++)
    if(A[x][y]){clues++;r=x*81-81+y*9-9+A[x][y];
      for(j=1;j<=Cols[r];j++){c1=Col[r][j];if(Uc[c1])return -1;Uc[c1]++; 
        for(k=1;k<=Rows[c1];k++){r1=Row[c1][k];Ur[r1]++;}}}

   ZeroMemory( V );
   for(c=1;c<=m;c++)
      {
      for(r=1;r<=Rows[c];r++)
         if(Ur[Row[c][r]]==0)
            V[c]++;
      }
 
   m0=0;m1=0;i=clues;solutions=0;
m2:i++;I[i]=0;min=n+1;if(i>81 || m0)goto m4;if(m1){C[i]=m1;goto m3;}
   for(c=1;c<=m;c++)if(!Uc[c]){if(V[c]<min){min=V[c];C[i]=c;if(min<2)goto m3;}} 
m3:c=C[i];I[i]++;if(I[i]>Rows[c])goto m4;
   r=Row[c][I[i]];if(Ur[r])goto m3;m0=0;m1=0;
   for(j=1;j<=Cols[r];j++){c1=Col[r][j];Uc[c1]++;}
   for(j=1;j<=Cols[r];j++){c1=Col[r][j]; 
      for(k=1;k<=Rows[c1];k++){r1=Row[c1][k];Ur[r1]++;if(Ur[r1]==1)
         for(l=1;l<=Cols[r1];l++){c2=Col[r1][l];V[c2]--;
            if(Uc[c2]+V[c2]<1)m0=c2;if(Uc[c2]==0 && V[c2]<2)m1=c2;}}} 
   nodes++;
   if(i==81)solutions++;if(solutions==smax)return smax;
   goto m2;
m4:i--;if(i<=clues)return solutions;c=C[i];r=Row[c][I[i]];
   for(j=1;j<=Cols[r];j++){c1=Col[r][j];Uc[c1]--; 
      for(k=1;k<=Rows[c1];k++){r1=Row[c1][k];Ur[r1]--;
         if(Ur[r1]==0)for(l=1;l<=Cols[r1];l++){c2=Col[r1][l];V[c2]++;}}}
   if(i>clues)goto m3;
   return solutions;
   }
